# -*- encoding: utf-8 -*-
##############################################################################
#
#    Copyright (C) 2008-2010 Anevia S.A. (www.anevia.com),
#    @author Raphaël Valyi <rvalyi@akretion.com>
#    @author Alexis de Lattre <alexis.delattre@akretion.com>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################


from osv import fields, osv
from datetime import datetime

#TODO make this configurable parameters:
fixed_month_init_day = 1
fixed_default_anniversary_month = 12

class stock_location(osv.osv):
    _inherit = "stock.location"

    def _is_under_maintenance(self, cr, uid, ids, field_name, arg, context=None):
        res = {}
        now = datetime.now()
        for fleet in self.browse(cr, uid, ids, context=context):
            if fleet.expire_time:
                res[fleet.id] = now < datetime.strptime(fleet.expire_time, '%Y-%m-%d')
            else:
                res[fleet.id] = False #by default no maintenance expire terms means no coverage
        return res

    def _expire_time(self, cr, uid, ids, field_name, arg, context=None):
        res = {}
        for fleet in self.browse(cr, uid, ids, context=context):
            max_time = 0
            for invoice_line in fleet.account_invoice_lines:
                if invoice_line.maintenance_end_date > max_time:
                    max_time = invoice_line.maintenance_end_date
            res[fleet.id] = max_time
        return res

    def _time_to_expire(self, cr, uid, ids, field_name, arg, context=None):
        res = {}
        now = datetime.now()
        for fleet in self.browse(cr, uid, ids, context=context):
            res[fleet.id] = fleet.expire_time and int((datetime.strptime(fleet.expire_time, '%Y-%m-%d') - now).days) or False
        return res

    #retrieve anniversary time from parent eventually
    def _anniversary_time(self, cr, uid, ids, field_name, arg, context=None):
        res = {}
        for fleet in self.browse(cr, uid, ids, context=context):
            res[fleet.id] = fleet.location_id and fleet.location_id.intrinsic_anniversary_time or fleet.intrinsic_anniversary_time
        return res

    def _check_expire_time(self, cr, uid, ids, context=None):
        ids_to_invalidate = []
        for line in self.pool.get('account.invoice.line').browse(cr, uid, ids, context=context):
            ids_to_invalidate.append(line.fleet_id.id)
        return ids_to_invalidate

    def _get_all_linked_locations(self, cr, uid, ids, context=None):
        """We need to invalidate both the stock location and it's subfleets"""
        subfleet_ids = self.search(cr, uid, [('location_id', 'in', ids)], context=context)
        return ids + subfleet_ids

    _columns = {
        'fleet_type': fields.selection([
            ('none','Not a Fleet'),
            ('fleet','Fleet'),
            ('sub_fleet','Sub Fleet')
            ], 'Fleet type'),
        'partner_id': fields.many2one('res.partner', 'Partner', ondelete='cascade'),
        'parent_partner_id': fields.related('location_id', 'partner_id', type='many2one', relation='res.partner', string='Partner of parent location', store={
            'stock.location': (_get_all_linked_locations, ['partner_id', 'location_id'], 10),
            }),
        'sale_order_lines': fields.one2many('sale.order.line', 'fleet_id', 'Sale order lines', readonly=True),
        'fleet_sale_order_lines': fields.one2many('sale.order.line', 'parent_fleet_id', 'Sale order lines', readonly=True),
        'account_invoice_lines': fields.one2many('account.invoice.line', 'fleet_id', 'Invoice lines', readonly=True),
        'fleet_account_invoice_lines': fields.one2many('account.invoice.line', 'parent_fleet_id', 'Invoice Lines', readonly=True),
        'crm_claims': fields.one2many('crm.claim', 'fleet_id', 'Related CRM claims', readonly=True),
        'fleet_crm_claims': fields.one2many('crm.claim', 'parent_fleet_id', 'Related CRM claims', readonly=True),
        'is_under_maintenance': fields.function(_is_under_maintenance, type='boolean', string="Is under maintenance"),
        'time_to_expire': fields.function(_time_to_expire, type='integer', string="Days before expiry"),
        'intrinsic_anniversary_time': fields.date('Intrinsic Time'),
        'anniversary_time': fields.function(_anniversary_time, type='date', string="Anniversary month", help="This is the month of anniversary of the maintenance contract. This field is a date field, but the only important information in this date is the month."), #TODO no year!
        'expire_time':fields.function(_expire_time, type='date', string="Maintenance expire time",
            store = {
                'account.invoice.line': (_check_expire_time,
                    ['fleet_id', 'maintenance_end_date', 'product_id'], 10),
                    # We also invalidate on product_id because it's onchange can trigger
                    # modifications of fleet_id and maintenance_end_date
            }),
    }

    def intrinsic_anniversary_time_change(self, cr, uid, ids, anniversary_time=False):
        result = {}
        if anniversary_time:
            anniversary_time = datetime.strptime(anniversary_time, '%Y-%m-%d')
            if anniversary_time.day != fixed_month_init_day:
                anniversary_time = datetime(anniversary_time.year, anniversary_time.month, fixed_month_init_day)
                result['value'] = {'intrinsic_anniversary_time': anniversary_time.strftime('%Y-%m-%d')}
                result['warning'] = {'title':'Incorrect anniversary date', 'message':"- Anniversary date should start at day %s of the month; corrected to day %s\n" % (fixed_month_init_day, fixed_month_init_day)}
        return result

    #FIXME: this call back is due to a current OpenERP limitation:
    #currently (v5, March 2009), it's not possible to use the orm create method to create a hierarchy of objects all together within a single call.
    #this is due to a bug/limitation of the Modified Pre-orderered Tree Traversal algorithm.
    def sub_fleet_change(self, cr, uid, ids, fleet_id):
        result = {}
        if not fleet_id:
            result['warning'] = {'title': 'Save PARENT FLEET first please!', 'message':'Due to a current OpenERP limitation, you should please close the SUBFLEET popup and save the form BEFORE adding any subfleet'}
            result['value'] = {'child_ids':[]}
        return result

    def _default_usage(self, cr, uid, context=None):
        if context is None:
            context = {}
        if context.has_key('fleet') or context.has_key('subfleet'):
            return 'customer'
        return False

    def _default_anniversary_time(self, cr, uid, context=None):
        default_anniversary_time = datetime(datetime.now().year, fixed_default_anniversary_month, fixed_month_init_day)
        return default_anniversary_time.strftime('%Y-%m-%d')

    def _default_fleet_type(self, cr, uid, context=None):
        if context is None:
            context = {}
        if context.get('fleet'):
            return 'fleet'
        elif context.get('subfleet'):
            return 'sub_fleet'
        return 'none'

    def _default_location_id(self, cr, uid, context=None):
        if context is None:
            context = {}
        if context.get('fleet'):
            res = self.pool.get('ir.property').get(cr, uid, 'property_stock_customer', 'res.partner', context=context).id
            return res
        if context.get('subfleet'):
            return context.get('parent_fleet_id') # This context key is set in the fleet form view, on the field 'child_ids'
        return False

    _defaults = {
        'usage': _default_usage,
        'intrinsic_anniversary_time': _default_anniversary_time,
        'fleet_type' : _default_fleet_type,
        'location_id' : _default_location_id,
    }

    _constraints = [] #TODO

stock_location()


class stock_picking(osv.osv):
    _inherit = "stock.picking"

    # copy extra invoice line info when invoicing on delivery,
    # if the "delivery" module is installed
    # ("fleet_maintenance" must depend on "delivery" for this to work properly !)
    # Note Alexis dL : I don't understand this comment... I don't think the delivery module
    # is required any more
    def _prepare_invoice_line(self, cr, uid, group, picking, move_line, invoice_id,
        invoice_vals, context=None):
        """ Write fleet_id field on product invoice lines"""
        il_vals = super(stock_picking, self)._prepare_invoice_line(cr, uid, group, picking,
            move_line, invoice_id, invoice_vals, context=context)
        if picking.sale_id and move_line.sale_line_id and move_line.sale_line_id.fleet_id:
            il_vals.update({'fleet_id': move_line.location_dest_id.id})
        return il_vals


    def _prepare_invoice(self, cr, uid, picking, partner, inv_type, journal_id, context=None):
        """ Copy Default Sub-fleet from sale order to invoice """
        invoice_vals = super(stock_picking, self)._prepare_invoice(cr, uid, picking, partner, inv_type, journal_id, context=context)
        if picking.sale_id and picking.sale_id.default_fleet_id:
            invoice_vals.update({'default_fleet_id': picking.sale_id.default_fleet_id.id})
        return invoice_vals

stock_picking()

