# -*- coding: utf-8 -*-
##############################################################################
#
#    Fleet maintenance module for OpenERP
#    Copyright (C) 2012 Anevia S.A.
#    @author David Beal <bealdavid@gmail.com>
#    @author Alexis de Lattre <alexis.delattre@akretion.com>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import fields, osv
import time
from tools.translate import _

class crm_claim_create_picking(osv.osv_memory):
    _name = "crm.claim.create.picking"
    _description = "Wizard to create pickings from the RMA form"

    _columns = {
        'source_location_id': fields.many2one('stock.location',
            'Source stock location', required=True,
            help="The move line of the picking that will be generated will have this source stock location."),
        'destination_location_id': fields.many2one('stock.location',
            'Destination stock location', required=True,
            help="The move line of the picking that will be generated will have this destination stock location."),
        'picking_type_label': fields.char('Picking type', size=20),
        'prodlot_id': fields.many2one('stock.production.lot', 'Serial Number',
            help="The move line of the picking that will be generated will have this production lot."),
        'partner_address_id': fields.many2one('res.partner.address',
            'Address', help="Address that will used in the picking that will be generated."),
        }


    def _get_field_infos(self, cr, uid, context=None):
        ''' define a dictionnary about picking fields '''

        picking_def = {
            'incoming': {'field': 'incoming_picking_id', 'picking_type': 'in'},
            'outgoing': {'field': 'outgoing_picking_id', 'picking_type': 'out'},
            'repair': {'field': 'repair_picking_id', 'picking_type': 'internal'},
            'in_supplier': {'field': 'in_supplier_picking_id', 'picking_type': 'out'},
            'out_supplier': {'field': 'out_supplier_picking_id', 'picking_type': 'in'},
            # WARNING : in_supplier is mvt Company -> Supplier, so it's an OUT
            # out_supplier is the mvt Supplier -> Company, so it's an IN
            }

        for rma_picking_type, caracteristics in picking_def.items():
            caracteristics['string'] = self.pool.get('crm.claim').fields_get(cr, uid, [caracteristics['field']], context=context)[caracteristics['field']]['string']

        return picking_def


    def default_get(self, cr, uid, fields, context=None):
        '''Fills up some default values on certain type of pickings'''
        claim_obj = self.pool.get('crm.claim')

        claim = claim_obj.browse(cr, uid, context['active_id'], context=context)

        fleet_id = claim.fleet_id.id

        picking_def = self._get_field_infos(cr, uid, context=context)

        res = {'picking_type_label': picking_def[context['rma_picking_type']]['string']}
        if context['rma_picking_type'] == 'incoming':
            res['source_location_id'] = fleet_id
        elif context['rma_picking_type'] == 'outgoing':
            res['destination_location_id'] = fleet_id

        if context['rma_picking_type'] in ['incoming', 'repair', 'in_supplier']:
            res['prodlot_id'] = claim.prodlot_id.id

        if context['rma_picking_type'] in ['in_supplier', 'out_supplier'] and claim.product_id.seller_id.id:
            address_delivery = self.pool.get('res.partner').address_get(cr, uid, [claim.product_id.seller_id.id], ['delivery'])

            if address_delivery:
                res['partner_address_id'] = address_delivery['delivery']
        if context['rma_picking_type'] in ['incoming', 'outgoing'] and claim.partner_address_id.id:
            res['partner_address_id'] = claim.partner_address_id.id

        return res


    def wiz_get_claim_serial(self, cr, uid, ids, context=None):
        ''' get serial number from claim and set in wizard '''

        claim = self.pool.get('crm.claim').browse(cr, uid, context['active_id'], context=context)
        self.write(cr, uid, ids[0], {'prodlot_id': claim.prodlot_id.id }, context=context)
        return True


    def wiz_set_stock_location(self, cr, uid, ids, context=None):
        '''Wizard to create pickings from the RMA form'''

        picking_def = self._get_field_infos(cr, uid, context=context)

        claim = self.pool.get('crm.claim').browse(cr, uid, context['active_id'], context=context)

        move_name = claim.incident_ref+':'+context['rma_picking_type']+':'+claim.product_id.name

        wiz = self.browse(cr, uid, ids[0], context=context)

        time_now = time.strftime("%Y-%m-%d %H:%M:%S")

        stock_move = {
            'name': move_name,
            'product_id': claim.product_id.id,
            'state': 'draft',
            'company_id': claim.company_id.id,
            'date': time_now,
            'date_expected': time_now,
            'product_qty': 1,
            'location_id': wiz.source_location_id.id,
            'location_dest_id': wiz.destination_location_id.id,
            'product_uom': claim.product_id.uom_id.id,
            'prodlot_id': wiz.prodlot_id.id,
            }

        stock_picking = {
            'invoice_state': 'none',
            'move_type': 'one',
            'type': picking_def[context['rma_picking_type']]['picking_type'],
            'origin': claim.incident_ref,
            'company_id': claim.company_id.id,
            'move_lines': [(0, 0, stock_move)],
            'address_id': wiz.partner_address_id.id,
            }

        picking_id = self.pool.get('stock.picking') \
            .create(cr, uid, stock_picking, context=context)

        self.pool.get('crm.claim').write(cr, uid, context['active_id'], \
            {picking_def[context['rma_picking_type']]['field']: picking_id }, context=context)

        return {'type': 'ir.actions.act_window_close'}
